% Phase Variance, mod 2pi, Second-Order Loop
%		plus statistics on skipped cycles.  A cycle is skipped when
%			absolute phase error exceeds 1 cycle.  At that point, the
%			nearest 1-cycle multiple becomes zero phase reference for skips.
%  SINE NONLINEARITY
% Toolboxes Required: control
% [23,000 points/minute, 26 seconds/10000 points (at 100 samples per output) with Athlon 3000+, sop=2]
clear;

% MODIFY PARAMETERS BELOW & GO
%*****************************
% Set either of the following parameters and set the other to zero.
  NS = 0.5; % N/S in Loop Bandwidth (noise required to prevent divide-by-zero)
  No = 0; % Noise Power Density, per Hz, to Signal Ratio
Sources = 2; % 1 or 2 independent noise sources
Phinit = 0;  % radian initial phase, usually = 0
Winit = 0; % radian/sec initial frequency, usually = 0
Wn = 1;      % rad/sec natural frequency
z = 1;    % damping factor
alpha = .97;   % Eq. (6.5)
SmpPerWL = 100; % ratio of sample rate to unity-gain radian bandwidth 
TshowXWL = 1; % (period between outputs) X (unity-gain radian bandwidth)
ncmpo = 5000;  % computed output points.
Offset = 0;     % offset at PD output (same sign as steady-state output)
truncatePh = 1; % 1:truncate phase; 0:do not
		% Choose Graphical Output
	%  sop=1:  Compute array
	%  sop=2:  Compute running stats (no array)
	%  sop=3:  Compute array and running stats
sop = 2;  % Set sop = 2 normally.
strtplt = 100; % skip first plotted points in statistical plots up to strtplt
%******************************

fprintf('\n\nPHASE STATISTICS (mod 2 pi)   NStat2(4/12/07)')
fprintf('\n2nd-order loop with SINUSOIDAL Phase Detector Characteristic')
fprintf('\nWn = %g rad/sec; zeta = %g; alpha = %g',Wn,z,alpha)
Bn = Wn*(1/(2*z) + 2*z*alpha^2)/4; % Noise BW in Hz
fprintf('\nNoise Bandwidth (linear), Bno = %g Hz.  Number of noise sources: %u\n',Bn,Sources)
if alpha ~= 1
    if alpha == 0
        kOverWn = 1/2/z;
    else
        kOverWn = (1+1/(1/alpha-1))/2/z;
    end
    fprintf('Offset is %g, producing %g rad/sec offset in center frequency\n',Offset,Offset*kOverWn*Wn)
else
    fprintf('Offset is %g, producing infinite offset in center frequency\n',Offset)
end

%Polynominals for open loop gain:
	N = [0     2*z*Wn*alpha   Wn^2]; % numerator
	D = [1   2*z*Wn*(1-alpha)   0 ]; % denominator

[a,b,c,d] = tf2ss(N,D); % elements of dynamic matrix equation
y = Phinit; % initial phase at output

%Compute unity-gain bandwidth
factor = N(2)^2-D(2)^2;
WL = sqrt((factor+sqrt(factor^2+4*N(3)^2))/2); % open-loop gain = 1

SmpRate = SmpPerWL*WL; % sample rate
OutInc = TshowXWL/WL;% period between outputs

% Compute multiplier of noise distribution having unity power.
if NS == 0,
	Dn = No;
	fprintf('Noise Density to Signal Ratio = %g /Hz',No)
else
	Dn = NS/Bn;
 	fprintf('Signal/Noise in Bno = %g',1/NS)
end
Kn = sqrt(Dn*SmpRate/2); 

last = 1 + ncmpo; % index of last output
ending = OutInc*ncmpo; % End time
SmpPerOut = round(SmpRate*OutInc); % samples per output period
t = linspace(0,ending,last); % linear sequence of times for plots
dt = (t(2) - t(1))/SmpPerOut; % sample time increment

if NS < 0.03 % less than 2% error from this approximation or from 14 terms
    sig2 = NS;
else
    sig2 = (pi^2)/3;
    for kk = 1:14
        sig2 = sig2 +4*((-1)^kk/kk^2)*besseli(kk,1/NS)/besseli(0,1/NS);
    end % for
end % if
fprintf('  Variance in Bno is So2 = %g rad^2',NS)
fprintf('\nTheoretical nonlinear variance, 1st order loop, same So2 in Bno: %g rad^2\n',sig2)

% To help judge inaccuracy of ignoring sampling,
%   compute extra phase shift produced at unity gain
SampPhShift = -(90/pi)*WL*OutInc/SmpPerOut; % radians lag from sampling at WL
fprintf('Added phase at unity O.L. gain due to sampling = %g deg.\n',SampPhShift)
if truncatePh
	fprintf('Phase will be shown in range 0.5 cycle, modulo 1 cycle.\n')	
else
	fprintf('Phase range will be unrestricted.\n')
end

if (NS ~= 0)
    Nstats = 1;
else
    Nstats = 0;
end % conditions OK for noise statistics
if ~Nstats
    fprintf('\nVariances not computed because of zero noise.\n')
else
    fprintf('\n')
end

%    BEGIN SIMULATION
NL = sin(-Phinit); % <<<<<<<<<<<< THE NON-LINEARITY
% state variables to give specified initial conditions
% X2 and X3 respectively in Eq. (6.50) and (6.51) with D(3) = 0:
x0(1) = (Winit-N(2)*(NL+Offset))/(N(3)-D(2)*N(2));
x0(2) = (Phinit-N(2)*x0(1))/N(3);
e = -Phinit; % initial phase error
freq(1) = Winit; % initial frequency at output
Nplus = 0; Nminus = 0; % skipped cycles
Fsum(1) = 0; Fnet(1) = 0; % rate of skipping

if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
	if e>0
		e = rem(e+pi,2*pi) - pi;
	else
		e = rem(e-pi,2*pi) + pi;
	end
end % if truncatePh
ersum = 0; ersum2 = 0;
msv(1) = NS; % start plot at linear value (avoid problem with ratio in dB)
if (sop ~= 2) 
	er(1) = e;
end
[Phi, Gamma] = c2d(a,b,dt); % values for use in solution eqution [see manual]

running = last; i = 1;
while running
    if (running > 10) %initialize arrays for better execution speed
        Fnet(last)=0;
        Fsum(last)=0;
        PDnoise(last)=0;
        freq(last)=0;
        mnn(last)=0;
        t(last)=0;
        tshrt(last)=0;
        msv(last)=0;
        varshrt(last)=0;
    end % preassigning arrays
	while i < last, % compute loop responses
        i = i + 1;
        for j = 1:SmpPerOut, % loop through unrecorded samples
            NL = sin(-y); % <<<<<<<<<<<< THE NON-LINEARITY 
            if Sources == 2,
                Noise1 = randn; Noise2 = randn;
                PDn = Kn*(Noise1*sin(y)+Noise2*cos(y));
            else
                PDn = Kn*randn;
            end	
            u1(1) = NL + Offset + PDn; % initial input to sampler
            u1(2) = u1(1); %  constant Hold output -> same at beginning and end of dt
            x = ltitr(Phi,Gamma,u1',x0); % solutions at period start and end [see manual] 
            yy = c*x'; % corresponding outputs (d is null for polynominals above)
            y = yy(2); % output at end of period is needed
            x0 = x(2,:)'; % final condition gives initial condition for next step
            eu = -y/(2*pi);
            if eu - Nplus + Nminus > 1
                Nplus = fix(eu + Nminus);
            elseif eu - Nplus + Nminus < -1
                Nminus = -fix(eu - Nplus);
            end
        end % for j
        PDnoise(i) = PDn;
		e = -y; % untruncated error
		if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
			if e>0
				e = rem(e+pi,2*pi) - pi;
			else
				e = rem(e-pi,2*pi) + pi;
			end
		end % if truncatePh
		if (sop ~= 2)
			er(i) = e;
		end
		if (sop == 2)||(sop == 3)
			ersum = ersum + e;
			ersum2 = ersum2 + e^2;
			msv(i) = ersum2/(i);
			mnn(i) = ersum/(i);
			Fsum(i) = (Nplus + Nminus)/(dt*SmpPerOut*(i-1));
			Fnet(i) = (Nplus - Nminus)/(dt*SmpPerOut*(i-1));
		end
		% corresponding frequency out from Eq. (6.43) with d3 = 0
		freq(i) = N(2)*u1(2) + x0(1)*(N(3)-N(2)*D(2)); 
	end % for i
	
	% END SIMULATION.  The rest is plotting.
	
	clf;
    ncmpo = last - 1;
	ending = ncmpo*OutInc;
	t = linspace(0,ending,last); % reestablish time vector

	%****** Start Statistical Plots
		tshrt = t(strtplt:last); varshrt = msv(strtplt:last);
		subplot(221);
		plot([t(strtplt) t(last)],[NS NS],'g',[t(strtplt) t(last)],[sig2 sig2],'m',tshrt,varshrt,'r')
		title('mean square phase (rad^2)')
		grid
		legend('linear','1ord theory','nonlinear',0)
		subplot(223);
        if Nstats
            plot(tshrt,10*log10(varshrt/NS),'b')
            title('rms phase ratio (dB)')
            grid
        end %if nstat
		subplot(222);
		plot(tshrt,mnn(strtplt:last),'k')
		title('mean phase error (rad)')
		grid
		subplot(224);
		plot(tshrt,Fnet(strtplt:last),'g',tshrt,Fsum(strtplt:last),'r')
		title('skips/sec, total and net')
		grid
		xlabel('seconds')
	%****** end plotting
    
    if (sop == 1)||(sop == 3) % compute from array
		arvar = std(er)^2;
		logarvar = 10*log10(arvar/NS);
		armnn = mean(er);
		fprintf('Estimate from error array:')
        fprintf('\n  output variance %g rad^2\n  ratio to So2 %g dB',arvar,logarvar);
		fprintf('\n  mean %g rad\n  %g points including initial phase\n',armnn,last);
    end % for error array stats
    
	if (sop == 2)||(sop == 3) % compute from running stats
        fprintf('Mean square phase error is variance if mean is known to be zero:')
        if Nstats
            variance = (msv(last)-(mnn(last))^2)*last/(last-1);
            fprintf('\n  mean square phase %g rad^2, phase variance %g rad^2',msv(last),variance);
        end
		fprintf('\n  %g skipped cycles with %g net skips',Nplus+Nminus,Nplus-Nminus);
		fprintf('\n  mean phase error %g rad',mnn(last));
        fprintf('\n  %g points (%g seconds) including initial phase\n',last,last*OutInc);
	end % for running stats
	
	sound(sin(6283*[0:1/8192:0.2]),8192); %0.2 seconds of 1 kHz tone;
	running = input('       Enter additional points (0 to quit, <0 for key in). ');
	if (running < 0)
		running = -running;
		fprintf('\n    Keyboard entry.  Type "return" to resume program.\n')
		keyboard
	end
	last = last + running;
end % while running
%Notes:
%Mean square phase plot: "linear" and "simple (FP)" (Fokker-Plank) are for 0 offset.
%Plotted ratio is (mean square phase out)/(variance in linear loop).