% Phase Variance, mod 2pi, Second-Order Loop with input phase modulation
%		plus statistics on skipped cycles.  A cycle is skipped when
%		absolute phase error exceeds 1 cycle.  At that point, the
%		nearest 1-cycle multiple becomes zero phase reference for skips. 
%  SINE NONLINEARITY
% Toolboxes Required: control
% [22,640 points/minute, 26.5 seconds/10000 points (sop=2 & 100 samples per output) with Athlon 3000+]
clear;

% MODIFY PARAMETERS BELOW & GO
%*****************************
% Set either of the following parameters and set the other to zero.
NS = .05; % N/S in Loop Bandwidth
No = 0; % Noise Power Density, per Hz, to Signal Ratio
Sources = 2; % 1 or 2 independent noise sources

% Set input phase modulation
PkDelPhi = 30; % degrees peak deviation, square wave
MPeriod = 8; % sample periods per modulation period, even number, not submultiple of SmpRate/K

Phinit = 0;  % radian initial phase
Winit = 0; % radian/sec initial frequency
Wn = 1;      % rad/sec natural frequency
z = .707;    % damping factor
alpha = 1;   % Eq. (6.5)
SmpPerWL = 100; % ratio of sample rate to unity-gain radian bandwidth 
TshowXWL = 1; % (period between outputs) X (unity-gain radian bandwidth)
ncmpo = 1000;  % computed output points.
Offset = 0;     % offset at PD output (same sign as steady-state output).  Unless 0, no variance computed.
truncatePh = 1; % 1:truncate phase; 0:do not
strtplt = 11; % skip first plotted points in statistical plots up to strtplt
	%  sop=1:  Compute array
	%  sop=2:  Compute running stats (no array)
	%  sop=3:  Compute array and running stats
sop = 2;  % Set sop = 2 normally.
%******************************

fprintf('\n\nPHASE STATISTICS (mod 2 pi) WITH HIGH-FREQUENCY PM   Sig2(4/13/07)')
fprintf('\n2nd-order loop with SINUSOIDAL Phase Detector Characteristic')
fprintf('\nWn = %g rad/sec; zeta = %g; alpha = %g',Wn,z,alpha)
Bn = Wn*(1/(2*z) + 2*z*alpha^2)/4; % Noise BW in Hz
fprintf('\nNoise Bandwidth = %g Hz.  Number of noise sources: %u\n',Bn,Sources)

%Polynominals for open loop gain:
	N = [0     2*z*Wn*alpha   Wn^2]; % numerator
	D = [1   2*z*Wn*(1-alpha)   0 ]; % denominator

[a,b,c,d] = tf2ss(N,D); % elements of dynamic matrix equation
y = Phinit; % initial phase at output

%Compute unity-gain bandwidth
factor = N(2)^2-D(2)^2;
WL = sqrt((factor+sqrt(factor^2+4*N(3)^2))/2); % open-loop gain = 1

SmpRate = SmpPerWL*WL; % sample rate
OutInc = TshowXWL/WL;% period between outputs

% Compute multiplier of noise distribution having unity power.
if NS == 0,
	Dn = No; NS = Bn*Dn;
	fprintf('Noise Density to Signal Ratio = %g /Hz',No)
else
	Dn = NS/Bn;
 	fprintf('Signal/Noise in Bno = %g',1/NS)
end
fprintf('\nOffset %g = asin(%g rad)',Offset,asin(Offset))
fprintf('\n%g degrees modulation at %g rad/sec.',PkDelPhi,SmpRate/MPeriod)
Kn = sqrt(Dn*SmpRate/2); 

last = 1 + ncmpo; % index of last output
ending = OutInc*ncmpo; % End time
SmpPerOut = SmpRate*OutInc; % samples per output period
t = linspace(0,ending,last); % linear sequence of times for plots
dt = (t(2) - t(1))/SmpPerOut; % sample time increment

PkDelRad = PkDelPhi*pi/180;
if NS < 0.03 % less than 2% error from this approximation or from 14 terms
    sigsqr = NS;
else
    sigsqr = (pi^2)/3;
    for kk = 1:14
        sigsqr = sigsqr +4*((-1)^kk/kk^2)*besseli(kk,1/NS)/besseli(0,1/NS);
    end % for
end % if
sigsqr = sigsqr/cos(PkDelRad);
fprintf('  Variance in Bno is So2 = %g rad^2',NS)
fprintf('\nTheoretical nonlinear variance, 1st order loop, same So2 in Bno: %g rad^2\n',sigsqr)

% To help judge inaccuracy of ignoring sampling,
%   compute extra phase shift produced at unity gain
SampPhShift = -(90/pi)*WL*OutInc/SmpPerOut; % radians lag from sampling at WL = K
fprintf('Added phase at unity O.L. gain due to sampling = %g deg.\n',SampPhShift)
if truncatePh
	fprintf('Phase will be shown in range 0.5 cycle, modulo 1 cycle.\n')	
else
	fprintf('Phase range will be unrestricted.\n')
end

if (Phinit == 0 && Offset == 0 && NS ~= 0)
    Nstats = 1;
else
    Nstats = 0;
end % conditions OK for noise statistics with known zero mean
if ~Nstats
    fprintf('\nVariances not computed because of zero noise or non-zero mean phase.\n')
else
    fprintf('\n')
end

%    BEGIN SIMULATION
NL = sin(-Phinit); % <<<<<<<<<<<< THE NON-LINEARITY
% state variables to give specified initial conditions
% X2 and X3 respectively in Eq. (6.50) and (6.51) with D(3) = 0:
x0(1) = (Winit-N(2)*(NL+Offset))/(N(3)-D(2)*N(2));
x0(2) = (Phinit-N(2)*x0(1))/N(3);
e = -Phinit; % initial phase error
freq(1) = Winit; % initial frequency at output
Nplus = 0; Nminus = 0; % skipped cycles
Fsum(1) = 0; Fnet(1) = 0; % rate of skipping

if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
	if e>0
		e = rem(e+pi,2*pi) - pi;
	else
		e = rem(e-pi,2*pi) + pi;
	end
end % if truncatePh
ersum = 0; ersum2 = 0;
var(1) = NS; % start plot at linear value (avoid problem with ratio in dB)
if (sop ~= 2) 
	er(1) = e;
end
[Phi, Gamma] = c2d(a,b,dt); % values for use in solution eqution [see manual]

running = last; i = 1;
while running
    if (running > 10)
        Fnet(last)=0;
        Fsum(last)=0;
        PDnoise(last)=0;
        freq(last)=0;
        mnn(last)=0;
        t(last)=0;
        tshrt(last)=0;
        var(last)=0;
        varshrt(last)=0;
    end % preassigning arrays
    ModBias = (MPeriod-1)/2;
	while i < last, % compute loop responses
		i = i + 1;
        Tsamples = (i-2)*SmpPerOut; % total samples used to here
        ModStart = Tsamples -.9 + round(MPeriod/4); % start modulation at 1/4 period
		for j = 1:SmpPerOut, % loop through unrecorded samples
			DelRad = PkDelRad * sign(ModBias - rem(j+ModStart,MPeriod));
			NL = sin(DelRad - y); % <<<<<<<<<<<< THE NON-LINEARITY 
			if Sources == 2,
                Noise1 = randn; Noise2 = randn;
                PDn = Kn*(Noise1*sin(y)+Noise2*cos(y));
            else
                PDn = Kn*randn;
            end				
			u1(1) = NL + Offset + PDn; % initial input to sampler
			u1(2) = u1(1); %  constant Hold output -> same at beginning and end of dt
			x = ltitr(Phi,Gamma,u1',x0); % solutions at period start and end [see manual] 
			yy = c*x'; % corresponding outputs (d is null for polynominals above)
			y = yy(2); % output at end of period is needed
			x0 = x(2,:)'; % final condition gives initial condition for next step
            eu = -y/(2*pi);
			if eu - Nplus + Nminus > 1
				Nplus = fix(eu + Nminus);
			elseif eu - Nplus + Nminus < -1
				Nminus = -fix(eu - Nplus);
			end
		end % for j
        PDnoise(i) = PDn;
		e = -y; % untruncated error
		if truncatePh, % truncate e range to pi % <<<<<<<<<<<<TRUNCATION
			if e>0
				e = rem(e+pi,2*pi) - pi;
			else
				e = rem(e-pi,2*pi) + pi;
			end
		end % if truncatePh
		if (sop ~= 2)
			er(i) = e + DelRad;
		end
		if (sop == 2)||(sop == 3)
			ersum = ersum + e;
			ersum2 = ersum2 + e^2;
			var(i) = ersum2/(i);
			mnn(i) = ersum/(i);
			Fsum(i) = (Nplus + Nminus)/(dt*(i-1));
			Fnet(i) = (Nplus - Nminus)/(dt*(i-1));
		end
		% corresponding frequency out from Eq. (6.43) with d3 = 0
		freq(i) = N(2)*u1(2) + x0(1)*(N(3)-N(2)*D(2));         
	end % for i
	
	% END SIMULATION.  The rest is plotting.
	
	clf;
    ncmpo = last - 1;
	ending = ncmpo*OutInc;
	t = linspace(0,ending,last); % reestablish time vector

%****** Start Statistical Plots
		tshrt = t(strtplt:last); varshrt = var(strtplt:last);
		if Nstats
            subplot(221);
            plot([t(strtplt) t(last)],[NS NS],'g',[t(strtplt) t(last)],[sigsqr sigsqr],'m',tshrt,varshrt,'r')
            title('phase variance (rad^2)')
            grid
            legend('linear','1st-order theoretical','nonlinear',0)
            subplot(223);
            plot(tshrt,10*log10(varshrt/NS),'b')
            title('variance ratio (dB)')
            grid
		end % zero initial conditions
		subplot(222);
		plot(tshrt,mnn(strtplt:last),'k')
		title('mean phase error (rad)')
		grid
		subplot(224);
		plot(tshrt,Fnet(strtplt:last),'g',tshrt,Fsum(strtplt:last),'r')
		title('skip frequency (skips/sec), total and net')
		grid
		xlabel('seconds')
	% end plotting
	
	if (sop == 1)||(sop == 3) % compute from array
        fprintf('\nEstimate from error array:')
        if Nstats
            arvar = std(er)^2;
            logarvar = 10*log10(arvar/NS);
        fprintf('\n  output variance %g rad^2\n  ratio to So2 %g dB',arvar,logarvar);
        end % if Nstats
        armnn = mean(er);
        fprintf('\n  mean phase error %g rad\n  %g points including initial phase\n',armnn,last);
	end % for error array stats
    
	if (sop == 2)||(sop == 3) % compute from running stats
       fprintf('\nRunning statistics for known zero mean:')
        if Nstats
            logvar = 10*log10(var(last)/NS);
            fprintf('\n  output variance %g rad^2\n  ratio to So2 %g dB',var(last),logvar);
        end
        fprintf('\n  %g skipped cycles with %g net skips',Nplus+Nminus,Nplus-Nminus);
		fprintf('\n  mean phase error %g rad\n  %g points (%g seconds) including initial phase\n',mnn(last),last,last*OutInc);	end % for running stats
	
	sound(sin(6283*[0:1/8192:0.2]),8192); %0.2 seconds of 1 kHz tone;
	running = input('    Enter additional points (0 to quit, <0 for key in). ');
	if (running < 0)
		running = -running;
		fprintf('\n    Keyboard entry.  Type "return" to resume program.\n')
		keyboard
	end
	last = last + running;
end % while running
