% Mean Time to First Skip in Noise, First-Order Loop
%   SINE NONLINEARITY
% Sampling rate must be high for accurate values with large noise.
%   At N/S = 5, SmpRate = 100*K produces F_bar about 5% low.
% [6 seconds for 100,000 increments at SmpRate=100*K, sop=2, Athlon 3000+]
% Toolboxes Required: none
clear;

% MODIFY PARAMETERS BELOW & GO
%*****************************
% Set either of the following 2 parameters and set the other to zero.
NS = .5; % N/S in Loop Bandwidth (1/rho)
No = 0; % Noise Power Density, per Hz, to Signal Ratio
Sources = 2; % 1 or 2 independent noise sources

Phinit = 0;  % radian initial phase
K = 1;      % rad/sec unity-gain frequency
SmpRate = 100*K; % sample rate
OutInc = 1/K;% period between outputs
ncmpo = 2000; % approximate output points (reduced by skips/2)
Offset = 0;     % offset at PD output (same sign as steady-state output)
		% Choose Graphical Output
strtplt = 1; % skip first plotted points in statistical plots up to strtplt
	%  sop=1:  Compute array
	%  sop=2:  Compute running stats (no time array to avoid max dim. limit)
	%  sop=3:  Compute array and running stats
sop = 2;  % Set sop = 2 normally.
%******************************

fprintf('\n\nMEAN TIME TO FIRST CYCLE SKIP (Tm)   Skip1(4/12/07)')
fprintf('\n1st-Order Loop with SINUSOIDAL Phase Detector Characteristic')
fprintf('\nK = %g rad/sec',K)
Bn =K/4; % Noise BW in Hz
fprintf('\nNoise Bandwidth (linear), Bno = %g Hz.  Number of noise sources: %u\n',Bn,Sources)

% Compute multiplier of noise distribution having unity power.
if NS == 0,
	Dn = No; NS = No*Bn;
	fprintf('Noise Density to Signal Ratio = %g /Hz\n',No)
else
	Dn = NS/Bn;
 	fprintf('Signal/Noise in Bno = %g\n',1/NS)
end
fprintf('Offset is %g, producing %g rad/sec offset in center frequency\n',Offset,Offset*K)

rho = 1/NS;
Kn = sqrt(Dn*SmpRate/2); 

last = 1 + ncmpo; % index of last output
ending = OutInc*ncmpo; % End time
SmpPerOut = SmpRate*OutInc; % samples per output period
dt = 1/SmpRate; % sample time increment
Tm = pi^2*rho*(besseli(0,rho))^2/(2*Bn);
fprintf('Theoretical Mean Time to First Skip with no offset: %g seconds.\n',Tm);
fprintf('                                               1/(TmBno): %g.\n',1/Tm/Bn);

% To help judge inaccuracy of ignoring sampling,
%   compute extra phase shift produced at unity gain
SampPhShift = -(90/pi)*K*OutInc/SmpPerOut; % radians lag from sampling at WL = K
fprintf('Added phase at unity O.L. gain due to sampling = %g deg.\n',SampPhShift)
fprintf('Phase will be reset to 0 and skip counted when outside 1 cycle.\n')	

dt = 1/SmpRate;
Nplus = 0; Nminus = 0; % skipped cycles
Tavg(1) = 0; % rate of skipping
Favg(1) = 0; % reciprocal of Tavg
Nrel(1) = 0; % relative differential in skips
time = 0; % time at last skip
%    BEGIN SIMULATION
running = last; i = 1; ii = 1; Nplus = 0; Nminus = 0; Tskips = 0;
Nsum(1) = 0; Ndif(1) = 0;	
er(1) = -Phinit;
freq(1) = -K*sin(Phinit); % initial frequency at output
while running
    if (running > 10) % initialize arrays to improve execution speed
        Favg(last)=0;
        FbarOver(last)=0;
        Fnet(last)=0;
        Fsum(last)=0;
        N(last)=0;
        Ndif(last)=0;
        Nsum(last)=0;
        PDnoise(last)=0;
        Tavg(last)=0;
        Tshrt(last)=0;
        Ttime(last)=0;
        freq(last)=0;
        mnn(last)=0;
        t(last)=0;
        tshrt(last)=0;
        var(last)=0;
        varshrt(last)=0;
    end % preassigning arrays
	while i < last
		NL = sin(-Phinit); % <<<<<<<<<<<< THE NON-LINEARITY
		y = Phinit; % initial phase error
		StartOver = 0; k = -1;
		while ~StartOver && (i < last), % compute loop responses
			i = i + 1; k = k + 1;
            for j = 1:SmpPerOut, % loop through unrecorded samples
                NL = sin(-y); % <<<<<<<<<<<< THE NON-LINEARITY 
                if Sources == 2,
                    Noise1 = randn; Noise2 = randn;
                    PDn = Kn*(Noise1*sin(y)+Noise2*cos(y));
                else
                    PDn = Kn*randn;
                end	
                u1 = NL + Offset + PDn; % initial input to sampler = constant Hold output -> same at beginning and end of dt
                y = y + u1*K*dt;
                if y<-2*pi
                    StartOver = 1;
                    Nplus = 1 + Nplus;
                elseif y>2*pi
                    StartOver = 1;
                    Nminus = Nminus + 1;
                end	
                if StartOver
                    time = dt*(j -.5 + k*SmpPerOut) + time;
                    break;
                end
            end % for j
			if StartOver % cycle was skipped and loop will begin again at initial conditions
				ii = ii + 1;
				Nsumm = Nplus + Nminus;
				Nsum(ii) = Nsumm;
				Ndif(ii) = Nplus - Nminus;
				if Nplus||Nminus
					Tavg(ii) = time/Nsumm;
					Favg(ii) = Nsumm/time;
				else
					Tavg(ii) = 0;
					Favg(ii) = 0;
				end
				Ttime(ii) = time;
			end
			e = -y; % untruncated error
			if (sop ~= 2)
				er(i) = e;
				freq(i) = K*u1; % corresponding frequency out from Eq. (6.43) with d3 = 0
				PDnoise(i) = PDn; 
			end
		end % while ~StartOver
	end % while i < last	
	% END SIMULATION.  The rest is plotting.
	
	ncmpo = last - 1;
	ending = ncmpo*OutInc;
    clf; % clear graphs
    
    %****** Start Mean Time Plots	
    if (strtplt < ii);
        Tshrt = Ttime(strtplt:ii);
        FbarOver = Favg/Bn;
        FOTheory = 1/(Bn*Tm);
        
        subplot(211);
        plot([Tshrt(1) Ttime(ii)],[FOTheory FOTheory],'g',Tshrt,FbarOver(strtplt:ii),'r')
        title(['F_b_a_r/Bno for S/N =',num2str(rho)]);
        grid
        xlabel('Simulated Time (seconds)')
        legend('theoretical','simulated',0)
        
        subplot(212);
        plot(Tshrt,Ndif(strtplt:ii),'om',Tshrt,Nsum(strtplt:ii),'+b')
        title('Total and Differential Skips')
        grid
        xlabel('Simulated Time (seconds)')
    end % plotting
    
    T21 = Tavg(ii);
    if T21
        Fbb0 = 1/T21/Bn;
    else
        Fbb0 = 0;
    end
    fprintf('\n  Average time to first skip is %g seconds, 1/(TmBno) = %g.',T21,Fbb0);
	fprintf('\n  %g total skips with plus exceeding minus by %g',Nsum(ii),Ndif(ii));
	fprintf('\n  %g output increments including initial phase',last);
	sound(sin(6283*[0:1/8192:0.2]),8192); %0.2 seconds of 1 kHz tone;
	running = input('\n    Enter additional points (0 to quit, <0 for key in). ');
	if (running < 0)
		running = -running;
		fprintf('\n    Keyboard entry.  Type "return" to resume program.\n')
		keyboard
	end
	last = last + running;
end % while running
